/***************************************************************


    Filename: interpreter.h
    Author: Cheng Wu Albert
    E-mail: bc_cwa@stu.ust.hk

    Description: The implementation of an interpreter for ScorAut

****************************************************************/

#ifndef __INTERPRETER
#define __INTERPRETER

#include <string>
#include <iostream>
#include <map>
#include <stack>
#include <queue>
#include <deque>
#include <string.h>
#include <stdio.h>

#include <math.h>

//definition of a standard "plugin" function prototype for the interpreter
#define FUNCTION(___funcname)		void ___funcname(void* __ps,int __argc) { \
											basicObj *__argv=new basicObj[__argc]; \
											for(int __i=0;__i<__argc;__i++){ \
												__argv[__argc-__i-1]=basicFunc::popStack(__ps); \
											}

									
								
#define ENDFUNCTION()				if(__argv) delete[] __argv;}

#define FUNCRETURN(___arg)			{  \
									basicFunc::pushStack(__ps,___arg); \
									if(__argv) delete[] __argv; \
									return; }


#define GETARG(___idx)				(__argv[___idx])



#include "bas.h"

#define typeError		-1
#define typeString		0
#define typeDouble		1
#define typeInteger		2
#define typeChar		3
#define typeFunction	4
#define typeOperand		5
#define typeObj			6
#define typeUnknown		7
#define typeArray		8
#define typeVarInst		9
#define typeSinglet		10

#define errOperandRep	"##operand.err"

#define errString		"<unknown string>"
#define errDouble		1.00
#define errInteger		1
#define errChar			' '
#define errFunction		basicFunc()


using namespace std;

typedef void (* basicFuncPtr)(void*,int);

class basicObj;


#define typeScriptSeq	100
#define typeCppFunc		101
#define typeUnknownFunc	102

//For anchoring the c++ functions to the script functions
class basicFunc
{
private:


	basicFuncPtr funcPtr;
	void *scriptSeqPtr;

	int typeID;
	string funcName;
public:
	basicFunc(string _funcName="",basicFuncPtr _funcPtr=NULL);
	void execute(void* _arg,int _argc ) const;
	void setFuncName(const string& _name);
	string getFuncName() const;
	void anchorFunc(const basicFuncPtr& _funcPtr);
	
	void createScriptSeq();
	void addScript(void* _scriptLine);
	void deleteScriptSeq();

	basicFuncPtr getFuncPtr() const;
	static basicObj popStack(void* _stack);
	static void pushStack(void *_stack,const basicObj& _bobj);
	
	~basicFunc();

};

basicFunc::~basicFunc()
{
	deleteScriptSeq();
}

basicFunc::basicFunc(string _funcName,basicFuncPtr _funcPtr)
{
	scriptSeqPtr=NULL;
	funcPtr=_funcPtr;
	funcName=_funcName;
	typeID=((funcPtr)?typeCppFunc:typeUnknownFunc);
}

void basicFunc::setFuncName(const string& _name)
{
	funcName=_name;
}

void basicFunc::anchorFunc(const basicFuncPtr& _funcPtr)
{
	typeID=typeCppFunc;
	funcPtr=_funcPtr;
}

string basicFunc::getFuncName() const
{
	return funcName;
}

basicFuncPtr basicFunc::getFuncPtr() const
{
	return funcPtr;
}


void basicFunc::execute(void *_arg,int _argc) const
{
	if(!funcPtr)
		return;

	(*funcPtr)(_arg,_argc); //execute the C++ function.
}
class basicObj;

class operand:public bas
{
public:
	string opname;
	int precedence;
	int argc;
	
	operand():opname(""),precedence(-1),argc(0){}
	operand operator=(const operand& _ocopy);
	operand(string _opname,int _precedence,int _argc):opname(_opname),precedence(_precedence),argc(_argc){}
	bool errOperand();
	
	//operator basicObj() const;
};



operand operand::operator =(const operand& _ocopy)
{
	opname=_ocopy.opname;
	precedence=_ocopy.precedence;
	argc=_ocopy.argc;
	return (*this);
}

bool operand::errOperand()
{
	return (opname==errOperandRep);
}

class basicArray;
class virtualMachine;

//a basic data storage object for the interpreter
class basicObj:public bas
{
	public:
	

		struct _data
		{

			string str;
			double dbl;
			int itg;
			char kar;
			basicFunc routine;
			basicArray *barr;		

		}data;

	int typeID;
	
	virtualMachine *machine;
	deque<basicObj>::iterator scope;



	static basicObj errBasicObjInst;

	basicObj(const basicObj& _ocopy);
	basicObj(int typeID=typeUnknown);
	~basicObj();
	basicObj(virtualMachine *_machine):machine(_machine){data.barr=NULL;}
	


	int getTypeID() const;
	int setTypeID(int _ID);
	void *getDataPtr();
	const void* getDataPtrConst() const;
	string toString() const;
	double toDouble() const;
	int toInteger() const;
	char toChar() const;

	basicObj& resolveTarget();
	basicObj& retrieveObj(const strVector& _path,bool _allowadd=false);	
	basicObj& retrieveObj(const string& _path,bool _allowadd=false);
	basicObj& retrieveLocalObj(const string& _path,bool _allowadd=false);
	basicObj& retrieveLocalObj(const strVector& _path,bool _allowadd=false);
	
	basicObj& addLocalObj(const string& _path,const basicObj& _obj);
	basicObj& addLocalObj(const strVector& _path,const basicObj& _obj);
	basicObj& addImmediateLocalObj(const string& _name,basicObj& _obj);
	
	bool removeLocalObj(const string& _path);
	bool removeLocalObj(const strVector& _path);
	bool removeImmediateLocalObj(const string& _name);


	void fork();
	void delArray();
	void setString(const string& _str);
	void setDouble(const double& _dbl);
	void setInteger(const int& _itg);
	void setChar(const char& _kar);
	void setFunction(const basicFunc& _func);
	void newArray();
	void setArrayElement(const string& _ky,const basicObj& _obj);
	void markAsVarInst();
	void setVarInst(const string& _name="",const string& _arrayIdx="",int _isArray=typeSinglet);

	void operator=(const basicObj& _ocopy);
	void operator=(const bool& _bvalue);
	void operator=(const int& _ivalue);
	void operator=(const double& _dvalue);
	void operator=(const string& _svalue);


	operator bool() const;
	operator int() const;
	operator double() const;
	operator string() const;

	basicObj operator*(const basicObj& _right) const;
	basicObj operator+(const basicObj& _right) const;
	basicObj operator-(const basicObj& _right) const;
	basicObj operator/(const basicObj& _right) const;
	basicObj operator%(const basicObj& _right) const;
	basicObj operator>>(const basicObj& _right) const; //pow
	basicObj operator<<(const basicObj& _right) const; //str_concat

	void operator*=(const basicObj& _right); 
	void operator+=(const basicObj& _right); 
	void operator-=(const basicObj& _right);
	void operator/=(const basicObj& _right); 
	void operator%=(const basicObj& _right); 
	void operator>>=(const basicObj& _right);//powAssign
	void operator<<=(const basicObj& _right); //str_concatAssign

	bool operator==(const basicObj& _cmpr) const;
	bool operator>(const basicObj& _cmpr) const;
	bool operator>=(const basicObj& _cmpr) const;
	bool operator<(const basicObj& _cmpr) const;
	bool operator<=(const basicObj& _cmpr) const;
	bool operator!=(const basicObj& _cmpr) const;
	bool operator!() const;

	void operator++(int);
	void operator--(int);

	bool isVarInst() const;
	bool isString() const;
	bool isDouble() const;
	bool isInteger() const;
	bool isArray() const;
	bool isOfDouble() const;
	
	string getString() const;
	double getDouble() const;
	int getInteger() const;
	char getChar() const;
	basicFunc getFunction() const;
	void execFunction(void * _stack,int _argc) const;
	basicArray &getArray();

	basicObj &getArrayElement(const string& _ky);
	
	operator operand() const;

	friend istream &operator <<(istream &is,const basicObj& obj);
};


class basicArray:public bas
{
private:
	//map<string,basicObj> arr;
	advMap<string,basicObj> arr;

	friend class basicObj;
};

bool basicObj::isArray() const
{
	return typeID==typeArray;
}

bool basicObj::isDouble() const
{
	return typeID==typeDouble;
}

bool basicObj::isOfDouble() const
{
	if(typeID==typeString)
		return getString().find(".")!=string::npos;

	return typeID==typeDouble;
}

bool basicObj::isString() const
{
	return typeID==typeString;
}

bool basicObj::isInteger() const
{
	return typeID==typeInteger;

}
basicObj::~basicObj()
{
	if(typeID==typeArray && data.barr)
			delete data.barr;
}


basicObj::basicObj(int _typeID)
{
	data.barr=NULL;
	typeID=_typeID;
}

basicObj::operator bool() const
{
	return (bool)toInteger();
}

basicObj::operator int() const
{
	return toInteger();
}

basicObj::operator double() const
{
	return toDouble();
}

basicObj::operator string() const
{
	return toString();
}


void basicObj::operator= (const bool& _bvalue)
{
	setInteger((int)_bvalue);
}

void basicObj::operator= (const int& _ivalue)
{
	setInteger(_ivalue);
}

void basicObj::operator= (const double& _dvalue)
{
	setDouble(_dvalue);
}

void basicObj::operator= (const string& _svalue)
{
	setString(_svalue);
}

void basicObj::operator+=(const basicObj& _right)
{
	(*this)=(*this)+_right;
}
void basicObj::operator-=(const basicObj& _right)
{
	(*this)=(*this)-_right;
}
void basicObj::operator*=(const basicObj& _right)
{
	(*this)=(*this)*_right;
}
void basicObj::operator/=(const basicObj& _right)
{
	(*this)=(*this)/_right;
}
void basicObj::operator%=(const basicObj& _right)
{
	(*this)=(*this)%_right;
}

void basicObj::operator>>=(const basicObj& _right)
{
	(*this)=(*this)>>_right;
}

void basicObj::operator<<=(const basicObj& _right)
{
	(*this)=(*this)<<_right;
}

basicObj basicObj::operator+(const basicObj& _right) const
{
	basicObj result;

	if(this->isOfDouble() || _right.isOfDouble())
		result.setDouble(this->toDouble()+_right.toDouble());
	else
		result.setInteger(this->toDouble()+_right.toDouble());

	return result;
	
}
basicObj basicObj::operator-(const basicObj& _right) const
{
	basicObj result;

	if(this->isOfDouble() || _right.isOfDouble())
		result.setDouble(this->toDouble()-_right.toDouble());
	else
		result.setInteger(this->toDouble()-_right.toDouble());

	return result;
}

basicObj basicObj::operator*(const basicObj& _right) const
{
	basicObj result;

	if(this->isOfDouble() || _right.isOfDouble())
		result.setDouble(this->toDouble()*_right.toDouble());
	else
		result.setInteger(this->toDouble()*_right.toDouble());

	return result;
}

basicObj basicObj::operator/(const basicObj& _right) const
{
	basicObj result;

	if(this->isOfDouble())
		result.setDouble(this->toDouble()/_right.toDouble());
	else
		result.setInteger(this->toDouble()/_right.toDouble());

	return result;
}


basicObj basicObj::operator%(const basicObj& _right) const
{
	basicObj result;

	result.setInteger(this->toInteger()%_right.toInteger());

	return result;
}

basicObj basicObj::operator>>(const basicObj& _right) const
{
	basicObj result;
	double re=pow(this->toDouble(),_right.toDouble());

	if(this->isOfDouble() || _right.isOfDouble() )
		result.setDouble(re);
	else
		result.setInteger(re);

	return result;
}

basicObj basicObj::operator<<(const basicObj& _right) const
{
	basicObj result;
	result=this->toString()+_right.toString();
	return result;
}

basicObj::operator operand() const
{
	operand o;
	o.opname=data.str;
//	cout<<"conv b->o:"<<(int)data.kar<<endl;
	o.precedence=data.kar;
	o.argc=data.itg;
	return o;
}


void basicObj::markAsVarInst()
{
	setTypeID(typeVarInst);

}

void basicObj::operator ++(int)
{
	double tmp=toDouble();
	tmp++;
	
	switch(typeID)
	{
	case typeInteger:
		setInteger(tmp);
		break;
	case typeDouble:
		setDouble(tmp);
		break;
	case typeString:
		setDouble(tmp);
		setString(toString());
		break;
	default:
		break;
	}
	
}


void basicObj::operator --(int)
{
	double tmp=toDouble();
	tmp--;
	
	switch(typeID)
	{
	case typeInteger:
		setInteger(tmp);
		break;
	case typeDouble:
		setDouble(tmp);
		break;
	case typeString:
		setDouble(tmp);
		setString(toString());
		break;
	default:
		break;
	}
	
}

bool basicObj::operator !=(const basicObj& _cmpr) const
{
	return !((*this)==_cmpr);
}

bool basicObj::operator <(const basicObj& _cmpr) const
{
	return toDouble()<_cmpr.toDouble();
}

bool basicObj::operator <=(const basicObj& _cmpr) const
{
	return toDouble()<=_cmpr.toDouble();
}


bool basicObj::operator >(const basicObj& _cmpr) const
{
	return toDouble()>_cmpr.toDouble();
}

bool basicObj::operator >=(const basicObj& _cmpr) const
{
	return toDouble()>=_cmpr.toDouble();
}

bool basicObj::operator !() const
{
	return !(bool)(*this);
}
bool basicObj::operator ==(const basicObj& _cmpr) const
{	
	if(isString() && _cmpr.isString())
		return data.str==_cmpr.data.str;
	else if(isArray())
	{
		if(!_cmpr.isArray())
			return false;

		return data.barr->arr==_cmpr.data.barr->arr;
	}
	else
		return toDouble()==_cmpr.toDouble();

}

void basicObj::setVarInst(const string& _name,const string& _arrayKey,int _isArray)
{

	markAsVarInst();

	if(_name=="")
		data.str+="|"+_arrayKey;
	else
		data.str=_name+"|"+_arrayKey;
	
	data.itg=_isArray;

	
}

void basicObj::setArrayElement(const string &_ky,const basicObj& _obj)
{
	if(getTypeID()!=typeArray || !data.barr)
	{
		newArray();
	}
	
	map<string,basicObj>::iterator i;
	i=data.barr->arr.find(_ky);
	if(i==data.barr->arr.end())
		data.barr->arr.insert(map<string,basicObj>::value_type(_ky,_obj));
	else
		(*i).second=_obj;
	
}

basicObj basicObj::errBasicObjInst(typeError);

basicObj& basicObj::getArrayElement(const string& _ky)
{
	if(getTypeID()!=typeArray || !data.barr)
		return basicObj::errBasicObjInst;

	
/*	map<string,basicObj>::iterator i;
	i=data.barr->arr.find(_ky);
	if(i==data.barr->arr.end())
		return basicObj::errBasicObjInst;
*/

	return data.barr->arr[_ky];
	//return (*i).second;

}


void basicObj::operator =(const basicObj& ocopy)
{
	typeID=ocopy.typeID;
	if(typeID==typeArray)
	{
		newArray();
		*(data.barr)=*(ocopy.data.barr);

	}
	else
		data=ocopy.data;
}

bool basicObj::isVarInst() const
{
	return (typeID==typeVarInst);
}
basicObj::basicObj(const basicObj& _ocopy)
{
	data.barr=NULL;
	typeID=_ocopy.typeID;
	if(typeID==typeArray)
	{
		newArray();
		*(data.barr)=*(_ocopy.data.barr);

	}
	else
		data=_ocopy.data;
}

void basicObj::delArray()
{
	if(typeID==typeArray && data.barr)
			delete data.barr;	
}


void basicObj::newArray()
{
	delArray();
	data.barr=new basicArray;
	setTypeID(typeArray);

}



ostream &operator<<(ostream &os,const basicObj& obj)
{
	string tp;
	switch(obj.getTypeID())
	{
	case typeString:
		tp="S";
		break;
	case typeInteger:
		tp="I";
		break;
	case typeDouble:
		tp="D";
		break;
	case typeOperand:
		tp="O";
		break;
	
	}
	os<<"<"<<tp<<">"<<obj.toString();
	return os;
}






void basicObj::execFunction(void* _stack,int _argc) const
{
	if(getTypeID()!=typeFunction)
		return;
	
	data.routine.execute(_stack,_argc);
}

void basicObj::setFunction(const basicFunc &_func)
{
	setTypeID(typeFunction);
	data.routine=_func;
}

basicFunc basicObj::getFunction() const
{
	if(getTypeID()!=typeFunction)
		return errFunction;

	return data.routine;
}

basicArray& basicObj::getArray()
{
	if(data.barr)
		return *(data.barr);
	//else die!
}
void basicObj::setString(const string&str)
{
	delArray();
	setTypeID(typeString);
	data.str=str;
}

void basicObj::setChar(const char& _kar)
{
	delArray();
	setTypeID(typeChar);
	data.kar=_kar;
}

void basicObj::setDouble(const double& _dbl)
{
	delArray();
	setTypeID(typeDouble);
	data.dbl=_dbl;
}

void basicObj::setInteger(const int& _itg)
{
	delArray();
	setTypeID(typeInteger);
	data.itg=_itg;
}

string basicObj::getString() const
{
	return toString();
}

int basicObj::getInteger() const
{
	return toInteger();
}

double basicObj::getDouble() const
{
	return toDouble();
}

char basicObj::getChar() const
{
	return toChar();
}

int basicObj::toInteger() const
{
	switch(typeID)
	{
	case typeString:
		return atoi(data.str.c_str());
	case typeInteger:
	case typeVarInst:
		return data.itg;
	case typeDouble:
		return (double)data.dbl;
	case typeChar:
		return (int)data.kar;
	default:
		return errInteger;
	}
}

double basicObj::toDouble() const
{
	switch(typeID)
	{
	case typeString:
	case typeVarInst:
		return atof(data.str.c_str());
	case typeInteger:
		return (double)data.itg;
	case typeDouble:
		return data.dbl;
	case typeChar:
		return (double)data.kar;
	default:
		return errInteger;
	}
}

char basicObj::toChar() const
{
	switch(typeID)
	{
	case typeString:
	case typeVarInst:
		if(data.str.length()>=1)
			return data.str.c_str()[0];
		else
			return errChar;
	case typeInteger:
		return (char)(data.itg%256);
	case typeDouble:
		return (char)((int)data.dbl)%256;
	case typeChar:
		return data.kar;
	default:
		return errChar;
	}
}


string basicObj::toString() const
{
	char tm[100];
	switch(typeID)
	{
	case typeOperand:
		sprintf(tm,"%s:%d,%d",data.str.c_str(),(int)data.kar,data.itg);
		return tm;
		break;
	case typeString:
	case typeVarInst:
		return data.str;
	case typeInteger:
		sprintf(tm,"%d",data.itg);
		return tm;
	case typeDouble:
		sprintf(tm,"%f",data.dbl);
		return tm;
	case typeChar:
		sprintf(tm,"%c",data.kar);
		return tm;

	default:
		return errString;

	}
}


int basicObj::getTypeID() const
{
	return typeID;
}


int basicObj::setTypeID(int _ID)
{
	int oldID=typeID;
	typeID=_ID;
	return oldID;
}


void *basicObj::getDataPtr()
{
	switch(typeID)
	{
		case typeString:
			return (void*)&(data.str);
		case typeInteger:
			return (void*)&(data.itg);
		case typeDouble:
			return (void*)&(data.dbl);
		case typeChar:
			return (void*)&(data.kar);
		case typeFunction:
			return (void*)&(data.routine);
		default:
			return (void*)NULL;
	}
}

const void* basicObj::getDataPtrConst() const
{
	switch(typeID)
	{
		case typeString:
			return (const void*)&(data.str);
		case typeInteger:
			return (const void*)&(data.itg);
		case typeDouble:
			return (const void*)&(data.dbl);
		case typeChar:
			return (const void*)&(data.kar);
		default:
			return (const void*)NULL;
	}	
}

class evalUnit:public bas
{
	private:
	void *unitData;
	int typeID;
	
	public:
	evalUnit(int _typeID=typeUnknown,void *_unitData=NULL);
	~evalUnit();
	void newOperand();
	void newObj();
	basicObj& getObj();
	operand& getOperand();

	
};




evalUnit::evalUnit(int _typeID,void *_unitData)
{
	typeID=_typeID;
	unitData=_unitData;
}

evalUnit::~evalUnit()
{
	if(!unitData)
		return;

	switch(typeID)
	{
		case typeObj:
			delete (basicObj*)unitData;
		break;
		case typeOperand:
			delete (operand*)unitData;
		break;
	}
}

void evalUnit::newOperand()
{
	if(typeID!=typeUnknown)
		return;

	operand *nop=new operand;
	typeID=typeOperand;
	
	unitData=(void*)nop;


}

void evalUnit::newObj()
{
	if(typeID!=typeUnknown)
		return;

	basicObj *nob=new basicObj;
	typeID=typeObj;
	
	unitData=(void*)nob;
}

basicObj& evalUnit::getObj()
{
	return *((basicObj*)unitData);
}

operand& evalUnit::getOperand()
{
	return *((operand*)unitData);

}

operand ffff;

//////////////////// CORE FUNCTIONS //////////////////////////////



FUNCTION(__core_opAdd) 
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);
	
	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=left_target+right_target;
		
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opSub)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=left_target-right_target;
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opMul)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=left_target*right_target;
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opDiv)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();


	result=left_target/right_target;

	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opPow)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=left_target>>right_target;
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opConc)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();
	
	result=left_target<<right_target;

	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opAssociate)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setString(left_target.toString()+":"+right_target.toString());
 
	FUNCRETURN(result);
	  
ENDFUNCTION()
	
FUNCTION(__core_opMarkVarInst)
	basicObj obj;
	obj=GETARG(0);
	obj.markAsVarInst();
ENDFUNCTION()

FUNCTION(__core_opPostfixInc)
	basicObj obj,result;
	obj=GETARG(0);
	basicObj& obj_target=obj.resolveTarget();
	
	result=obj_target;
	obj_target++;

	FUNCRETURN(result);


ENDFUNCTION()

FUNCTION(__core_opPostfixDec)
	basicObj obj,result;
	obj=GETARG(0);
	basicObj& obj_target=obj.resolveTarget();
	
	result=obj_target;
	obj_target--;

	FUNCRETURN(result);


ENDFUNCTION()

FUNCTION(__core_opPrefixInc)
	basicObj obj,result;
	obj=GETARG(0);
	basicObj& obj_target=obj.resolveTarget();
	
	obj_target++;
	result=obj_target;

	FUNCRETURN(result);


ENDFUNCTION()

FUNCTION(__core_opPrefixDec)
	basicObj obj,result;
	obj=GETARG(0);
	basicObj& obj_target=obj.resolveTarget();
	
	obj_target--;
	result=obj_target;

	FUNCRETURN(result);


ENDFUNCTION()


FUNCTION(__core_opLarger)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target>right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opSmaller)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target<right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opSmallerEqual)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target<=right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opLargerEqual)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target>=right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opEqual)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target==right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opNotEqual)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result.setInteger(left_target!=right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opCondAnd)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=((bool)left_target&&(bool)right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opCondOr)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=((bool)left_target||(bool)right_target);
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opCondXor)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=(bool_xor((bool)left_target,(bool)right_target));
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opAddAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target+=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opMulAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target*=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opDivAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target*=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opSubAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target-=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opPowAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	if(left_target.isDouble() || right_target.isDouble())
	{
		double d=pow((double)left_target.toDouble(),(double)right_target.toDouble());
		left_target.setDouble(d);
	}
	else
	{
		int i=pow((double)left_target.toInteger(),(double)right_target.toInteger());
		left_target.setInteger(i);
	}

	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opMod)
	basicObj left,right,result;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	result=left_target%right_target;
 
	FUNCRETURN(result);

ENDFUNCTION()

FUNCTION(__core_opModAssign)
	basicObj left,right;
	left=GETARG(0);
	right=GETARG(1);

	basicObj &left_target=left.resolveTarget();
	basicObj &right_target=right.resolveTarget();

	left_target%=right_target;
 
	FUNCRETURN(left_target);

ENDFUNCTION()

FUNCTION(__core_opInt)
	basicObj left,result;
	left=GETARG(0);


	basicObj &left_target=left.resolveTarget();

	result=left_target.getInteger();
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opNegate)
	basicObj left,result;
	left=GETARG(0);


	basicObj &left_target=left.resolveTarget();

	result=!left_target;
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opDouble)
	basicObj left,result;
	left=GETARG(0);


	basicObj &left_target=left.resolveTarget();

	result=left_target.getDouble();
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opBool)
	basicObj left,result;
	left=GETARG(0);


	basicObj &left_target=left.resolveTarget();

	result=(bool)left_target;
	
	FUNCRETURN(result);
	  
ENDFUNCTION()

FUNCTION(__core_opString)
	basicObj left,result;
	left=GETARG(0);


	basicObj &left_target=left.resolveTarget();

	result=left_target.getString();
	
	FUNCRETURN(result);
	  
ENDFUNCTION()
/////////////////////////////////////////////////////////////////////////




class virtualMachine:public bas
{
	public: 
		typedef basicObj vbasicObj;
		typedef operand voperand;
		typedef basicFunc vbasicFunc;
		typedef evalUnit vevalUnit;
		typedef deque<vbasicObj>::iterator scopeItr;


	private:
	
	advMap<string,operand> opmap;
	//advMap<string,vbasicObj> root;
	

	deque<vbasicObj> scopes; //begin()=> global
	scopeItr	currentScope;
	

	stack<vbasicObj> evalStack;
	stack<voperand> opStack;
	queue<vbasicObj> expr;
	int cargc; //current arg count


	public:
	
	virtualMachine();
	
	vector<vbasicObj> digestExpression (string _expr);
	vector<vbasicObj> infix2Postfix(const vector<vbasicObj>& _infix);
	void evalPostfix(const vector<vbasicObj>& _postfix);

	
	void printType() const;

	bool addVar(const string &_key,const vbasicObj &_val);
	bool removeVar(const string &_key);
	void emptyVars();

	/*obsolete*/ void compileExpression(const string& _expr);

	int getUnitType(const string& _expr);
	int detectTokenType(int _c);
	bool workState(int &_state,basicObj& _dst,char _c,int &_posAdj,bool _startnewState,bool _term);
	void recordStateItem(queue<vbasicObj>& _dst,const vbasicObj &_item);

	void initializeSystemVarsAndFuncs();
	void initializeSystemOpMap();
	void anchorFunction(scopeItr _dstScope,string _funcName,basicFuncPtr _fptr);
	vbasicObj& retrieveObj(scopeItr _scope,const string& _name,bool _allowaadd=false);
	vbasicObj& retrieveObj(scopeItr _scope,const strVector& _name,bool _allowaadd=false);

};




#define stateUnknown	0
#define stateString		1
#define stateEscaped	2
//#define stateInteger	3
//#define stateDouble	4
#define stateDigit		4
#define stateOperand	5
#define stateChar		15

#define tokenStringBound	1
#define tokenDigit			4
#define tokenSymbol		    5
#define tokenChar			15
#define tokenEscape			2
#define tokenUnknown		0

#define precOpen		0
#define precSep			1
#define precAssign		2
#define precCondOr		3
#define precCondAnd		4
#define precLogOr		5
#define precLogXor		6
#define precLogAnd		7
#define precEquality	8
#define precRelational	9
#define precShift		10
#define precAdd			11
#define precMul			12
#define precPow			13
#define precUnary		14
#define precPrimary		15
#define precAssociate	16
#define precHyperPrimary	precAssociate


#define precClose		125

#define opUndef			-1
#define opZary			0
#define opUnary			1
#define opBinary		2
#define opTenary		3
#define opVarary		-2

int virtualMachine::detectTokenType(int _c)
{
	char c2[2];
	c2[1]='\0';
	c2[0]=_c;
	switch(_c)
	{
	case '\\':

	return tokenEscape;
		break;
	case '"':
	return tokenStringBound;
		break;
	case '0':case '1':case '2':case '3':case '4':case '5':case '6':case '7':case '8':case '9':case '.':
	return tokenDigit;
		break;
	default:
		if(&(this->opmap[c2])==&(this->opmap.errorObj))
			return tokenChar;
		else
			return tokenSymbol;

	}

	return tokenUnknown;
}


bool virtualMachine::workState(int &_state,basicObj& _dst,char _c,int &_posAdj,bool _startnewState,bool _term)
{

	
	
	bool saveThis=false;
	
	operand opt;

	int type_token=detectTokenType(_c);
	char st[2];
	st[1]='\0';
	st[0]=_c;
	string tmp;

	if(_startnewState)
		_dst.setString("");


	switch(_state)
	{
	case stateChar:
	// a variant of string without escape support and string bound
	if(type_token!=stateChar || _term)
	{
		_state=typeUnknown;

		saveThis=true;
		
		_state=type_token;
		
		if(_state!=stateString)
		{
			
			_posAdj--;
		}
		break;
	}

	((string*)_dst.getDataPtr())->append((string)st);

		break;
	case stateString:
	

		if(type_token==tokenStringBound || _term)
		{
			_state=stateUnknown;
			
			saveThis=true;
			break;
		}else if(type_token==tokenEscape)
		{
			_state=stateEscaped;
			//stateNoChange=true;
			break;
		}

	((string*)_dst.getDataPtr())->append((string)st);
		break;
	
	case stateEscaped:
	((string*)_dst.getDataPtr())->append((string)ESCMAP[st]);
		//drop back to stateString
		_state=stateString;
		//stateNoChange=true;
		break;

	case stateOperand:
	

	tmp=_dst.toString()+st;
	
	if(this->opmap[tmp].errOperand() || _term)
	{
		//err!

		_state=stateUnknown;
		saveThis=true;
		_posAdj--;
		
		opt=this->opmap[_dst.toString()];
	//	cout<<"prece=="<<(opt.precedence%256)<<endl;
		_dst.data.kar=((char)(opt.precedence%256));
		_dst.data.itg=(opt.argc);
		_dst.setTypeID(typeOperand);
		
		break;


	}
	((string*)_dst.getDataPtr())->append((string)st);
	
	
		break;

	case stateDigit:
	if(type_token!=tokenDigit || _term)
	{
		if(_dst.getString().find(".")==string::npos)
			_dst.setInteger(_dst.toInteger());
		else
			_dst.setDouble(_dst.toDouble());
		
		saveThis=true;
		_state=type_token;
		_posAdj--;
		break;
	}
	((string*)_dst.getDataPtr())->append((string)st);		
		break;

	case stateUnknown:
		saveThis=false;
		_state=type_token;
		
		if(_state!=stateString)
		{
			
			_posAdj--;
		}

		break;
	}
	
	return saveThis;
}

void virtualMachine::recordStateItem(queue<virtualMachine::vbasicObj>& _dst,const basicObj &_item)
{
	
}

vector<virtualMachine::vbasicObj> virtualMachine::infix2Postfix(const vector<virtualMachine::vbasicObj> &_infix)
{
	vector<virtualMachine::vbasicObj> postfix;
	
	vector<virtualMachine::vbasicObj>::const_iterator i;
	stack<virtualMachine::vbasicObj> tmpstack;
	
	operand op;
	int nb;
	//nb=0;
	
	for(i=_infix.begin();i!=_infix.end();i++)
	{	
		
		switch((*i).getTypeID())
		{
		case typeOperand:
		//	cout<<"typeo:"<<(*i)<<endl;

			op=(operand)(basicObj)(*i);
			
			if(op.precedence==precOpen)
			{
	//			cout<<"op=0"<<endl;
				//case '(' or '['
				while(!tmpstack.empty() && (((operand)(basicObj)tmpstack.top()).precedence==precAssociate ))
				{
					postfix.push_back(tmpstack.top());
					tmpstack.pop();
				}
				tmpstack.push((*i));

			}
			else if(op.precedence==precClose)
			{
		//		cout<<"op=255"<<endl;
				//case ')' or ']'
				nb=1;

				while(!tmpstack.empty() && ((operand)(basicObj)tmpstack.top()).precedence>0)
				{
					
					op=(operand)(basicObj)(tmpstack.top());
					if(op.precedence==precSep)
						nb++;


					postfix.push_back(tmpstack.top());
					tmpstack.pop();
					
					//nb++;
				}

				if(!tmpstack.empty())
					tmpstack.pop();

				vbasicObj opp=(*i);
				opp.data.itg=nb;

				postfix.push_back(opp);

			}
			else
			{
				while(!tmpstack.empty() && op.precedence<=((operand)(basicObj)tmpstack.top()).precedence)
				{
					postfix.push_back(tmpstack.top());
					tmpstack.pop();
				}
				tmpstack.push((*i));

			}
			break;
		default:
			//just write out to the postfix expr;
			
			postfix.push_back((*i));
			break;

		}
	}
	

	while(!tmpstack.empty())
	{
			postfix.push_back(tmpstack.top());
			tmpstack.pop();
	}


	cout<<"result of i->p: \n++++++++++++\n";
	for(i=postfix.begin();i!=postfix.end();i++)
	{
		cout<<(*i)<<" ";
	}

	cout<<"\n++++++++++++++\n";
	return postfix;
}

vector<virtualMachine::vbasicObj> virtualMachine::digestExpression(string _expr)
{
	vector<virtualMachine::vbasicObj> vmb;
	int state=stateUnknown;

	_expr+="#";
	int i;
	int len;
	len=_expr.length();
	basicObj bo;
	bool term;
	bool startnew=true;
	
	for(i=0;i<len;i++)
	{
		char c;
		c=_expr[i];
		
		startnew=workState(state,bo,c,i,startnew,term=(i==len-1));
		


	//	cout<<"sss:"<<bo<<endl;
		if(startnew)
		{	
			vmb.push_back(bo);
		}
		
		if(term) break;


	}
	
			//vmb.push(bo);

	cout<<"result:\n**************\n";

	for(vector<virtualMachine::vbasicObj>::iterator vit=vmb.begin();
		vit!=vmb.end();
		vit++)
		{
			cout<<(*vit)<<endl;
		}
	cout<<"************\n";
	return vmb;
}


/* prev

	DEF_OPERAND("(",precOpen,0);
	DEF_OPERAND(")",precClose,0);

	DEF_OPERAND("[",precOpen,0);
	DEF_OPERAND("]",precClose,0);


	

	
	DEF_OPERAND("++",precClose-2,1);
	DEF_OPERAND("--",precClose-2,1);



	DEF_OPERAND(",",precSep,2);



	DEF_OPERAND("=",precOpen+2,2);
	DEF_OPERAND("==",precOpen+2,2);
	DEF_OPERAND(">=",precOpen+2,2);
	DEF_OPERAND(">",precOpen+2,2);
	DEF_OPERAND("<=",precOpen+2,2);
	DEF_OPERAND("<",precOpen+2,2);
	DEF_OPERAND("!=",precOpen+2,2);	

	DEF_OPERAND("+",precOpen+3,2);
	DEF_OPERAND("-",precOpen+3,2);
	DEF_OPERAND("~",precOpen+3,2); //string_concat

	DEF_OPERAND("*",precOpen+4,2);
	DEF_OPERAND("/",precOpen+4,2);

	DEF_OPERAND("^",precOpen+5,2);

	DEF_OPERAND("%",precClose-1,2);
	DEF_OPERAND("$",precClose-1,1);
	DEF_OPERAND("@",precClose-1,1);
	

  */

#define DEF_OPERAND(___OPNAME,___PREC,___ARGC,___CPPFUNC) opmap.setVal(___OPNAME,operand(___OPNAME,___PREC,___ARGC)); \
												anchorFunction(currentScope,___OPNAME,___CPPFUNC);

virtualMachine::vbasicObj &virtualMachine::retrieveObj(virtualMachine::scopeItr _scope,const string& _name,bool _allowaadd)
{
	strVector v;
	v=translateAddr(_name);
	return (*_scope).retrieveObj(v);
}

virtualMachine::vbasicObj &virtualMachine::retrieveObj(virtualMachine::scopeItr _scope,const strVector& _name,bool _allowaadd)
{
	return (*_scope).retrieveObj(_name);
}

void virtualMachine::anchorFunction(virtualMachine::scopeItr _dstScope,string _funcName,basicFuncPtr _fptr)
{
	basicObj fu;
	basicFunc fprstr(_funcName,_fptr);
	fu.setFunction(fprstr);

	(*_dstScope).setArrayElement(_funcName,fu);
	
}


void virtualMachine::initializeSystemVarsAndFuncs()
{
	basicObj fobj;
	/*anchorFunction(currentScope,"+",&__core_opAdd);
	anchorFunction(currentScope,"~",&__core_opConc);
	anchorFunction(currentScope,"/",&__core_opDiv);
	anchorFunction(currentScope,"*",&__core_opMul);
	anchorFunction(currentScope,"^",&__core_opPow);
	anchorFunction(currentScope,"-",&__core_opSub);*/
}

void virtualMachine::initializeSystemOpMap()
{
	opmap.setErrorObject(operand(errOperandRep,-1,-3));

	DEF_OPERAND("(",precOpen,opZary,NULL); //
	DEF_OPERAND(")",precClose,opZary,NULL); // 

	DEF_OPERAND("[",precOpen,opZary,NULL); //
	DEF_OPERAND("]",precClose,opZary,NULL); //
	DEF_OPERAND(":[",precOpen,opZary,NULL); //

	
	
	
	DEF_OPERAND("++",precUnary,opUnary,__core_opPostfixInc); //
	DEF_OPERAND("--",precUnary,opUnary,__core_opPostfixDec); //
	DEF_OPERAND("^+",precUnary,opUnary,__core_opPrefixInc); //
	DEF_OPERAND("^-",precUnary,opUnary,__core_opPrefixDec); //
	
	DEF_OPERAND("!",precUnary,opUnary,__core_opNegate); //
//	DEF_OPERAND("~",precUnary,opUnary,NULL);
	DEF_OPERAND("<int>",precUnary,opUnary,__core_opInt); // 
	DEF_OPERAND("<double>",precUnary,opUnary,__core_opDouble); // 
	DEF_OPERAND("<string>",precUnary,opUnary,__core_opString); //
	DEF_OPERAND("<bool>",precUnary,opUnary,__core_opBool); //

	DEF_OPERAND(",",precSep,opZary,NULL); //



	DEF_OPERAND("=",precAssign,opBinary,&__core_opAssign);  //
	DEF_OPERAND("+=",precAssign,opBinary,&__core_opAddAssign); //
	DEF_OPERAND("*=",precAssign,opBinary,&__core_opMulAssign); //
	DEF_OPERAND("/=",precAssign,opBinary,&__core_opDivAssign); //
	DEF_OPERAND("-=",precAssign,opBinary,&__core_opSubAssign); //
	DEF_OPERAND("^=",precAssign,opBinary,&__core_opPowAssign); //
	DEF_OPERAND("%=",precAssign,opBinary,&__core_opModAssign); //

	DEF_OPERAND("==",precEquality,opBinary,&__core_opEqual); //
	DEF_OPERAND("!=",precEquality,opBinary,&__core_opNotEqual); //

	DEF_OPERAND(">=",precRelational,opBinary,&__core_opLargerEqual); //
	DEF_OPERAND(">",precRelational,opBinary,&__core_opLarger); // 
	DEF_OPERAND("<=",precRelational,opBinary,&__core_opSmallerEqual); //
	DEF_OPERAND("=>",precRelational,opBinary,&__core_opLargerEqual); //
	DEF_OPERAND("=<",precRelational,opBinary,&__core_opSmallerEqual); //
	DEF_OPERAND("<",precRelational,opBinary,&__core_opSmaller); //
	

	DEF_OPERAND("+",precAdd,opBinary,&__core_opAdd); //
	DEF_OPERAND("-",precAdd,opBinary,&__core_opSub); //
	DEF_OPERAND("@",precAdd,opBinary,&__core_opConc); //string_concat

	DEF_OPERAND("*",precMul,opBinary,&__core_opMul); //
	DEF_OPERAND("/",precMul,opBinary,&__core_opDiv); //
	DEF_OPERAND("%",precMul,opBinary,&__core_opMod); // 


//	DEF_OPERAND(">>",precShift,opBinary,NULL);
//	DEF_OPERAND("<<",precShift,opBinary,NULL);


	DEF_OPERAND("^",precPow,opBinary,&__core_opPow); //

	DEF_OPERAND("&&",precCondAnd,opBinary,&__core_opCondAnd); //
	DEF_OPERAND("||",precCondOr,opBinary,&__core_opCondOr); //
	DEF_OPERAND("|",precCondOr,opBinary,&__core_opCondOr); //
	DEF_OPERAND("!|",precCondOr,opBinary,__core_opCondXor); //

	DEF_OPERAND("&",precPrimary,opVarary,NULL);
	DEF_OPERAND("$",precPrimary,opUnary,__core_opMarkVarInst); //

	DEF_OPERAND(":",precAssociate,opBinary,__core_opAssociate); //

}


virtualMachine::virtualMachine()
{
	
//	root.setErrorObject(vbasicObj::errBasicObjInst);
	cout<<"creating vm\n";
	basicObj glb;
	glb.newArray();

	cout<<"scopes init vm\n";
	scopes.push_back(glb);
	cout<<"scopes init-end vm\n";
	currentScope=scopes.begin();

	//initialize some system vars and funcs
	cout<<"initVars vm\n";
	initializeSystemVarsAndFuncs();
	initializeSystemOpMap();

	cout<<"endall vm\n";
	///////////////////////////////////////


	
	


}

void virtualMachine::compileExpression(const string& _expr)
{
    vector<string> lexunits;
    lexunits=str_split(_expr," ");
    vector<string>::iterator i;
    string cur;
    int len;
	operand cop;
    int stringParseState=0;
    string curParsedString,sprefix;
    	  double tmpd;
    for(i=lexunits.begin();i!=lexunits.end();i++)
    {	
		vbasicObj bo;
       cur=(*i);
       len=cur.length();
       


       if(cur[0]=='"' || stringParseState==1)
       {
          if(stringParseState==0)
          {
            curParsedString="";
            stringParseState=(cur[len-1]!='"' || len==1)?1:2;
            sprefix="";

            
          }else if(stringParseState==1)
          {
            stringParseState=(cur[len-1]!='"')?1:2;
			if(len>1 && cur[len-2]=='\\' && stringParseState==2)
				stringParseState=1;

            sprefix=" ";                       
			
          }
		  curParsedString.append(sprefix+evalEscapedString(cur));
		  
		  
       }
	   
	   else if(isdigit(cur[0]) || ((cur[0]=='-' || cur[0]=='+' || cur[0]=='.' ) && len>1))
	   {
		   tmpd=valOf(cur);
		   if(cur[0]=='.')
			   cur.insert(0,"0");

		   if(cur.find(".") != string::npos)
		   {
				//double
			   bo.setDouble(tmpd);
		   }
		   else
		   {
			   bo.setInteger(tmpd);
				//integer
		   }
	
		  
	   }
	   else
	   {
			cop=opmap[cur];
			bo.setString(cur);
			
			if(cop.opname!=errOperandRep)
				bo.setTypeID(typeOperand);	
			
	   }
		

	
		if(stringParseState==2)
	   {
			bo.setString(curParsedString);
			stringParseState=0;
		}

		if(stringParseState!=1)
		{
		cout<<"pushing "<<bo<<endl;
		 expr.push(bo);
		}
    }

    
}
int virtualMachine::getUnitType(const string& _expr)
{
	return typeUnknown;
}

void virtualMachine::printType() const
{
		bas::printType("VM");
}


/*bool virtualMachine::addVar(const string &_key,const vbasicObj &_val)
{
	if(root.find(_key)==root.end())
		return false;

	root.insert(map<string,vbasicObj >::value_type(_key,_val));
	
	return true;
}

bool virtualMachine::removeVar(const string &_key)
{
	if(root.find(_key)==root.end())
		return false;

	root.erase(_key);
	return true;
}

void virtualMachine::emptyVars()
{

	root.clear();
}
*/

basicObj basicFunc::popStack(void *_pstack)
{
	stack<basicObj>* p=(stack<basicObj>*)_pstack;
	basicObj ob=p->top();
	cout<<"poping "<<ob<<"\n";
	p->pop();
	return ob;
}

void basicFunc::pushStack(void *_pstack,const basicObj& _bobj)
{
	stack<basicObj>* p=(stack<basicObj>*)_pstack;
	p->push(_bobj);
}

void basicFunc::createScriptSeq()
{
	typeID=typeScriptSeq;
	deleteScriptSeq();
	scriptSeqPtr=new vector<vector<basicObj> >;

}

void basicFunc::addScript(void *_scriptLine)
{
	((vector<vector<basicObj> >*)scriptSeqPtr)->push_back(*((vector<basicObj>*)_scriptLine));

}

void basicFunc::deleteScriptSeq()
{
	if(scriptSeqPtr)
		delete scriptSeqPtr;
}

basicObj& basicObj::retrieveObj(const string& _path,bool _allowaadd)
{
	if(!machine)
		return basicObj::errBasicObjInst;

	return machine->retrieveObj(scope,translateAddr(_path),_allowadd);
}

basicObj& basicObj::retrieveObj(const strVector& _path,bool _allowaadd)
{
	if(!machine)
		return basicObj::errBasicObjInst;

	return machine->retrieveObj(scope,_path,_allowaadd);

}

basicObj& basicObj::retrieveLocalObj(const strVector& _path,bool _allowaadd)
{
	strVector::const_iterator i;
	basicObj* cobj=this;
	for(i=_path.begin();i!=_path.end();i++)
	{
		if(cobjcobj->getTypeID()!=typeArray)
		{
			if(_allowaadd)
			{
				cobj->newArray();
				cobj->addImmediateLocalObj((*i),(string)"");
				
			}
			else
				return basicObj::errBasicObjInst;
			
		}
		cobj=&(cobj->getArrayElement((*i)));
	}
	return (*cobj);
}

basicObj& basicObj::retrieveLocalObj(const string& _path,bool _allowaadd)
{
	strVector vpath;
	vpath=translateAddr(_path);

	strVector::const_iterator i;
	basicObj* cobj=this;
	for(i=vpath.begin();i!=vpath.end();i++)
	{
		if(cobj->getTypeID()!=typeArray)
			return basicObj::errBasicObjInst;

		cobj=&(cobj->getArrayElement((*i)));
	}
	return (*cobj);
}

bool basicObj::removeLocalObj(const strVector& _path)
{
	strVector::const_iterator i,j;

	basicObj* cobj=this;
	for(i=_path.begin();i!=_path.end();i++)
	{
		if(cobj->getTypeID()!=typeArray)
			return false;
		
		j=i;
		if(++j==_path.end())
			break;

		cobj=&(cobj->getArrayElement((*i)));
	}

	return cobj->removeImmediateLocalObj((*i));
}

bool basicObj::removeLocalObj(const string& _path)
{
	strVector vpath;
	vpath=translateAddr(_path);
	return removeLocalObj(vpath);


}


bool basicObj::removeImmediateLocalObj(const string& _name)
{
	if(!data.barr)
		return false;

	return data.barr->arr.delVal(_name);

	
}



basicObj& basicObj::resolveTarget()
{
	if(isVarInst())
	{
		return retrieveObj(data.str);
	}

	return (*this);
	
}

/*operand::operator basicObj() const
{
	basicObj j;
	j.setChar((char)(precedence%256));
	j.setInteger(argc);
	j.setString(opname);
	j.setTypeID(typeOperand);

}*/


typedef virtualMachine interpreter;

#endif